##
## wod_4: Observation influence estimation by bootstrap
##
##
## 


##
## Auxiliar function, that creates bootstrap set from the total set
##

getBootstrap <- function( data = "data.frame" ){
  
  bootstrap_data <- data
  
  boot_indexes   <- sample(1:nrow(data),nrow(data),replace=TRUE) 
  
    
  bootstrap_data[1:nrow(data),] <- data[boot_indexes,]

  return(bootstrap_data)
   
}  


## Function wod_4
##
##                  performance metric : concordance index

##  Arguments:      data  : SurvivalDataSet object
##                  nruns  : number of runs of the bootstrap
##                  layer5_model  : the layer5 predictor


##  Output   :      vector with length equal to the number of data rows , each position with the estimated influence on concordance
##



wod_4 <-  function( dataset = "SurvivalDataSet" , nruns   ) {
  
  

  actual_data <- cbind( dataset@data , dataset@time_vector , dataset@status_vector )
  
  time_index   <- ncol(actual_data) - 1
  status_index <- ncol(actual_data)  
   
  
  data_length  <- nrow(actual_data)
  
  obs_influences <- rep(x = 0 , data_length)
  
  obs_max_influences <- rep(x= 0 , data_length)
  
  concs_matrix  <- matrix( data = 0 , nrow = nruns, ncol = data_length  );
   
  
  ## calculate the baseline concordance, according to the baseline (all observations)
  #
  cox_object <- coxph( Surv(actual_data[,time_index], as.integer(actual_data[,status_index]) ) ~ .   , data = actual_data[,-c(time_index,status_index)] )
  baseline_concordance <- cox_object$concordance[1]/(cox_object$concordance[1] + cox_object$concordance[2] )   

  cat("baseline:" , baseline_concordance )

  for( row_index in 1:data_length ){
  
    concordance_sum <- 0
    
    coxph.call <- function () {
      boot_data  <- getBootstrap(data = actual_data[-row_index,])
      actual_conc  <- tryCatch({
        #
        cox_object <- coxph( Surv(boot_data[,time_index], as.integer(boot_data[,status_index]) ) ~ .,
                             data = boot_data[,-c(time_index,status_index)],
                             control = coxph.control(iter.max = 1000, outer.max = 1000), ties = 'efron')
  
          cox_object$concordance[1]/(cox_object$concordance[1] + cox_object$concordance[2] )
      }, error = function(e) {
        cat('Error! trying to resample 10 times')
        return(NULL)
      })
      return(actual_conc)
    }
    
    for ( i in 1:nruns ){
      actual_conc <- coxph.call()
      if (is.null(actual_conc)) {
        for( ix in 1:10) {
          cat('Error: ', ix)
          actual_conc <- coxph.call()
          if (!is.null(actual_conc)) {
            break
          }
        }
      }
      if (is.null(actual_conc)) {
        next
      }
      concordance_run <- actual_conc - baseline_concordance
      
      concordance_sum <- concordance_sum + concordance_run
      
      concs_matrix[i,row_index] <- concordance_run
    }
    
    obs_influences[row_index] <- concordance_sum/nruns ;
    
    sorted_concs <- sort( concs_matrix[,row_index] )
    
    obs_max_influences[row_index] <- sorted_concs[ nruns ] #max value
    
    cat("influences:",obs_influences[row_index],"\n")
 }  
  
 
 
 ## calculate p-value of hypothesis test : H0:= delta<=0 ,. H1: delta>0
 #
 pvalue_vector <- rep(x= 0, data_length)
 
 for( j in 1:data_length ){
   
   sorted_concs <- sort( concs_matrix[,j] )
   
   index = 1
   while( index<=nruns && sorted_concs[index] <= 0  ){
     
     index = index +1
   }
   
   pvalue_vector[j] <- 1 -  (nruns-index)/(nruns)
   
 }

 out <- data.frame(cbind(1:data_length, obs_influences,obs_max_influences, pvalue_vector))
 row.names(out) <- row.names(actual_data)
 colnames(out) <- c("id","exp infl", "max" , "p-value")
 output <- out[ order(out[,4]) , ] 
 
 return( output )
  
}







