##
## wod_4hist: Observation influence estimation by bootstrap , returns the matrix with all measures
##
##
## 


## Function wod_4hist
##
##                  performance metric : concordance index

##  Arguments:      data  : SurvivalDataSet object
##                  nruns  : number of runs of the bootstrap
##                  layer5_model  : the layer5 predictor


##  Output   :      vector with length equal to the number of data rows , each position with the estimated influence on concordance
##

wod_4hist_k <-  function( dataset = "SurvivalDataSet" , nruns  , m ) {
  
  
  
  actual_data <- cbind( dataset@data , dataset@time_vector , dataset@status_vector )
  
  time_index   <- ncol(actual_data) - 1
  status_index <- ncol(actual_data)  
  
  
  data_length  <- nrow(actual_data)
  
  obs_influences <- rep(x = 0 , data_length)
  
  obs_max_influences <- rep(x= 0 , data_length)
  
  concs_matrix  <- matrix( data = 0 , nrow = nruns, ncol = data_length  );
  
  
  ## calculate the baseline concordance, according to the baseline (all observations)
  #
  cox_object <- coxph( Surv(actual_data[,time_index], as.integer(actual_data[,status_index]) ) ~ .   , data = actual_data[,-c(time_index,status_index)] )
  baseline_concordance <- cox_object$concordance[1]/(cox_object$concordance[1] + cox_object$concordance[2] )   
  
  cat("baseline:" , baseline_concordance )
  
  for( row_index in 1:data_length ){
    
    concordance_sum <- 0
    
    coxph.call <- function () {
      boot_data <- getBootstrap_K(data = actual_data[-row_index,], k = m)
      actual_conc  <- tryCatch({
        #
        cox_object <- coxph( Surv(boot_data[,time_index], as.integer(boot_data[,status_index]) ) ~ .,
                             data = boot_data[,-c(time_index,status_index)],
                             control = coxph.control(iter.max = 1000, outer.max = 1000), ties = 'efron')
        
        cox_object$concordance[1]/(cox_object$concordance[1] + cox_object$concordance[2] )
      }, error = function(e) {
        cat('Error! trying to resample 10 times')
        return(NULL)
      })
      return(actual_conc)
    }
    
    for ( i in 1:nruns ){
      
      actual_conc <- coxph.call()
      if (is.null(actual_conc)) {
        for( ix in 1:10) {
          cat('Error: ', ix)
          actual_conc <- coxph.call()
          if (!is.null(actual_conc)) {
            break
          }
        }
      }
      if (is.null(actual_conc)) {
        next
      }
      
      concordance_run <- actual_conc - baseline_concordance
      
      concordance_sum <- concordance_sum + concordance_run
      
      concs_matrix[i,row_index] <- concordance_run     
    }
    
    
    cat(  "Antidote Bootstrap: " , row_index , "out of", data_length , " complete \n")
  }  
  
  
  
  return(concs_matrix)
  
}





